import io

from pygt3x.bit_pack_acceleration import BitPackAcceleration
from pygt3x.componenets import AccelerationSample


class Activity1Payload:
    """
    Class for Activity 1 Payload
    """

    def __init__(self, payload_bytes, timestamp):
        self.AccelerationSamples = BitPackAcceleration.unpack_activity(
            payload_bytes, timestamp, True
        )


class Activity2Payload:
    """Class for Activity 2 Payload."""

    @staticmethod
    def unpack_activity2(payload_bytes, timestamp):
        stream = io.BytesIO(payload_bytes)
        sample = [0, 0, 0]

        for _ in range(0, int(len(payload_bytes) / 6)):
            for axis in range(0, 3):
                b = stream.read(2)
                sample[axis] = int.from_bytes(b, byteorder="little", signed=True)
            yield AccelerationSample(timestamp, sample[0], sample[1], sample[2])

    def __init__(self, payload_bytes, timestamp):
        self.AccelerationSamples = self.unpack_activity2(payload_bytes, timestamp)


class Activity3Payload:
    """
    Class for Activity 3 Payload
    """

    def __init__(self, payload_bytes, timestamp):
        self.AccelerationSamples = BitPackAcceleration.unpack_activity(
            payload_bytes, timestamp, False
        )



"""Class for unpacking activity."""
from pygt3x.componenets import AccelerationSample


class BitPackAcceleration:
    @staticmethod
    def unpack_activity(source, timestamp: int, flip_xy: bool):
        """
        Unpacks activity stored as sets of 3, 12-bit integers

        Parameters:
        source (byte array): Activity payload bytes array
        flipXY (bool): If true sample is extracted in Y,X,Z order.
         Otherwise X,Y,Z order.

        Returns:
        Generator which produces acceleration samples as Int16 values

        """
        offset = 0
        index = -1
        sample = [0, 0, 0]

        def get_next_byte():
            nonlocal index
            index += 1
            if index == len(source):
                return None
            return source[index]

        current = 0
        while True:
            for axis in range(0, 3):
                if (offset & 0x7) == 0:
                    current = get_next_byte()
                    if current is None:
                        return

                    shifter = (current & 0xFF) << 4
                    offset += 8

                    current = get_next_byte()
                    if current is None:
                        return

                    shifter |= (current & 0xF0) >> 4
                    offset += 4
                else:
                    shifter = (current & 0x0F) << 8
                    offset += 4
                    current = get_next_byte()
                    if current is None:
                        return

                    shifter |= current & 0xFF
                    offset += 8

                if 0 != (shifter & 0x0800):
                    shifter |= 0xF000

                if shifter > 32767:
                    shifter -= 65535
                sample[axis] = shifter
            if flip_xy:
                yield AccelerationSample(
                    timestamp, x=sample[1], y=sample[0], z=sample[2]
                )
            else:
                yield AccelerationSample(
                    timestamp, x=sample[0], y=sample[1], z=sample[2]
                )
